import * as React from "react";

import {
  BaseFocusable as BaseFocusableInterface,
  BoundingRect,
} from "./BaseFocusable.interface";

import { createReactRef } from "@applicaster/zapp-react-native-utils/reactUtils";
import { focusManager } from "@applicaster/zapp-react-native-utils/appUtils/focusManager";
import * as FOCUS_EVENTS from "@applicaster/zapp-react-native-utils/appUtils/focusManager/events";
import { noop } from "@applicaster/zapp-react-native-utils/functionUtils";

type ScrollDirection = FocusManager.IOS.Direction | FocusManager.Web.Direction;

type Props = {
  initialFocus?: boolean;
  id: string;
  groupId?: string;
  preferredFocus?: boolean;
  onRegister?: (focusable: FocusManager.TouchableRef) => void;
  onUnregister?: (focusable: FocusManager.TouchableRef) => void;
  willReceiveFocus?: FocusManager.FocusEventCB;
  hasReceivedFocus?: FocusManager.FocusEventCB;
  willLoseFocus?: FocusManager.FocusEventCB;
  hasLostFocus?: FocusManager.FocusEventCB;
  failedLostFocus?: FocusManager.FocusEventCB;
  onPress?: (nativeEvent?: Nullable<React.MouseEvent>) => void;
  onLongPress?: (nativeEvent?: Nullable<React.MouseEvent>) => void;
  onPressIn?: (nativeEvent?: Nullable<React.MouseEvent>) => void;
  onPressOut?: (nativeEvent?: Nullable<React.MouseEvent>) => void;
  onFocus?: FocusManager.FocusEventCB;
  onBlur?: FocusManager.FocusEventCB;
  selected?: boolean;
  skipFocusManagerRegistration?: boolean;
};

export class BaseFocusable<
  T extends Props = Props
> extends BaseFocusableInterface<T> {
  constructor(props) {
    super(props);
    const { initialFocus } = this.props;

    this.state = {
      focused: initialFocus || false,
    };

    this.ref = createReactRef();
  }

  componentDidMount() {
    this._isMounted = true;
    const component = this;

    const { skipFocusManagerRegistration } = this.props;

    if (skipFocusManagerRegistration) {
      return;
    }

    focusManager.register({
      id: this.getId(),
      component: component,
    });

    const { initialFocus } = this.props;

    if (initialFocus) {
      focusManager.setFocus(this.getId());
    }
  }

  getId(props?: { id: string; groupId: string }) {
    const { id } = props || this.props;

    return id;
  }

  getRect() {
    let boundingRect: BoundingRect = {};

    if (this.ref && this.ref.current) {
      boundingRect = this.ref.current.getBoundingClientRect();
    }

    return boundingRect;
  }

  isMounted() {
    return this._isMounted;
  }

  componentWillUnmount() {
    this._isMounted = false;

    const { skipFocusManagerRegistration } = this.props;

    if (skipFocusManagerRegistration) {
      return;
    }

    focusManager.unregister(this.getId(), { group: this.isGroup || false });
  }

  componentDidUpdate(prevProps) {
    const { skipFocusManagerRegistration } = this.props;

    if (skipFocusManagerRegistration) {
      return;
    }

    if (prevProps?.id && this.props?.id && prevProps.id !== this.props.id) {
      focusManager.unregister(this.getId(prevProps), {
        group: this.isGroup || false,
      });
    }
  }

  measureView() {}

  onRegister = (focusable?: FocusManager.FocusableRef) => {
    const { onRegister = noop } = this.props;
    onRegister(focusable);
  };

  onUnregister = (focusable?: FocusManager.FocusableRef) => {
    const { onUnregister = noop } = this.props;
    onUnregister(focusable);
  };

  willLoseFocus: FocusManager.FocusEventCB = (focusable, scrollDirection) => {
    const { willLoseFocus = noop } = this.props;
    willLoseFocus(focusable, scrollDirection);

    focusManager.invokeHandler?.(
      FOCUS_EVENTS.WILL_LOSE_FOCUS,
      this,
      scrollDirection
    );
  };

  willReceiveFocus: FocusManager.FocusEventCB = (
    focusable,
    scrollDirection
  ) => {
    const { willReceiveFocus = noop } = this.props;
    willReceiveFocus(focusable, scrollDirection);

    focusManager.invokeHandler?.(
      FOCUS_EVENTS.WILL_RECEIVE_FOCUS,
      this,
      scrollDirection
    );
  };

  /**
   * will invoke the underlying component's focus method
   * @param {Object} focusable - sender
   * @param {Object} scrollDirection
   * @returns {Promise}
   */
  onFocus: FocusManager.FocusEventCB = (focusable, scrollDirection) => {
    const { onFocus = noop } = this.props;
    this.setFocusedState(true);
    onFocus(focusable, scrollDirection);

    focusManager.invokeHandler?.(FOCUS_EVENTS.FOCUS, this, scrollDirection);
  };

  /**
   * will invoke the underlying component's hasReceivedFocus method
   * @param {Object} focusable - sender
   * @param {Object} scrollDirection
   * @returns {Promise}
   */
  hasReceivedFocus: FocusManager.FocusEventCB = (
    focusable,
    scrollDirection
  ) => {
    const { hasReceivedFocus = noop } = this.props;
    hasReceivedFocus(focusable, scrollDirection);

    focusManager.invokeHandler?.(
      FOCUS_EVENTS.HAS_RECEIVED_FOCUS,
      this,
      scrollDirection
    );
  };

  /**
   * will invoke the underlying component's hasLostFocus method
   * @param {Object} focusable - sender
   * @param {Object} scrollDirection
   * @returns {Promise}
   */
  hasLostFocus: FocusManager.FocusEventCB = (focusable, scrollDirection) => {
    const { hasLostFocus = noop } = this.props;
    hasLostFocus(focusable, scrollDirection);

    focusManager.invokeHandler?.(
      FOCUS_EVENTS.HAS_LOST_FOCUS,
      this,
      scrollDirection
    );
  };

  /**
   * will invoke the underlying component's failedLostFocus method
   * @param {Object} focusable - sender
   * @param {Object} scrollDirection
   * @returns {Promise}
   */
  failedLostFocus: FocusManager.FocusEventCB = (
    focusable: FocusManager.FocusableRef | undefined,
    scrollDirection?: ScrollDirection
  ) => {
    const { failedLostFocus = noop } = this.props;
    failedLostFocus(focusable, scrollDirection);
  };

  /**
   * will invoke the underlying component's onBlur method
   * @param {Object} focusable - sender
   * @param {Object} scrollDirection
   * @returns {Promise}
   */
  onBlur: FocusManager.FocusEventCB = (focusable, scrollDirection) => {
    const { onBlur = noop } = this.props;
    this.setFocusedState(false);
    onBlur(focusable, scrollDirection);
    focusManager.invokeHandler?.(FOCUS_EVENTS.BLUR, this, scrollDirection);
  };

  onPress = (keyEvent?: Nullable<React.MouseEvent>) => {
    const { onPress = noop } = this.props;
    onPress(keyEvent);
  };

  /**
   * will invoke the underlying component's onLongPress method
   * @param {Object} keyEvent
   * @returns {Promise}
   */

  onLongPress = (keyEvent?: Nullable<React.MouseEvent>) => {
    const { onLongPress = noop } = this.props;
    onLongPress(keyEvent);
  };

  onPressOut = (keyEvent?: Nullable<React.MouseEvent>) => {
    const { onPressOut = noop } = this.props;
    onPressOut(keyEvent);
  };

  onPressIn = (keyEvent?: Nullable<React.MouseEvent>) => {
    const { onPressIn = noop } = this.props;
    onPressIn(keyEvent);
  };

  isInGroup(group: FocusManager.TouchableRef) {
    const { groupId } = this.props;
    const { id } = group.props;

    return id === groupId;
  }

  setFocusedState = (focused) => {
    if (this._isMounted) {
      this.setState({ focused });
    }
  };
}
