import * as React from "react";
import * as R from "ramda";
import { usePickFromState } from "@applicaster/zapp-react-native-redux/hooks";
import { useNavigation } from "@applicaster/zapp-react-native-utils/reactHooks/navigation";
import { useTheme } from "@applicaster/zapp-react-native-utils/theme";

import {
  getNavigationProps,
  resolveNavigationPlugin,
  routeIsPlayerScreen,
} from "@applicaster/zapp-react-native-utils/navigationUtils";

import { getBackgroundImageUrl } from "../utils";
import { LayoutContainer } from "./LayoutContainer";
import { NavBarContainer } from "./NavBarContainer";
import { ScreenContainer } from "./ScreenContainer";

import { ScreenLayoutContextProvider } from "./ScreenLayoutContextProvider";
import { isApplePlatform } from "@applicaster/zapp-react-native-utils/reactUtils";
import { PathnameContext } from "../../../Contexts/PathnameContext";
import { useNavbarState } from "@applicaster/zapp-react-native-utils/reactHooks/screen/useScreenContext";
import { ScreenDataContext } from "../../../Contexts/ScreenDataContext";

type Components = {
  NavBar: React.ComponentType<any>;
  Background: React.ComponentType<any>;
};

type ComponentsExtraProps = {
  Background?: Record<string, any>;
  NavBar?: Record<string, any>;
};

type Props = {
  Components: Components;
  ComponentsExtraProps?: ComponentsExtraProps;
  children: React.ReactNode;
};

const Layout = ({ Components, ComponentsExtraProps, children }: Props) => {
  const navigator = useNavigation();
  const theme = useTheme();
  const { title, visible } = useNavbarState();
  const { activeRiver, currentRoute } = navigator;

  const {
    appState: { appReady = false } = {},
    remoteConfigurations,
    plugins = [],
  } = usePickFromState(["appState", "remoteConfigurations", "plugins"]);

  const navigationProps = getNavigationProps({
    navigator,
    title,
    category: "menu",
  });

  const backgroundColor = React.useMemo(() => theme.app_background_color, []);

  const backgroundImageUrl = React.useMemo(
    () => getBackgroundImageUrl(remoteConfigurations),
    [remoteConfigurations]
  );

  const menuPlugin = React.useMemo(
    () =>
      resolveNavigationPlugin({
        category: "menu",
        navigations: activeRiver?.navigations,
        plugins,
      }) as QuickBrickPlugin,
    []
  );

  const NavBar = React.useMemo(
    () => Components.NavBar || (menuPlugin.module as React.ComponentType<any>),
    []
  );

  const [navBarReady, setNavBarReady] = React.useState(false);

  const navBarContainer = (
    <NavBarContainer isVisible={visible} onReady={setNavBarReady}>
      <NavBar
        ref={ComponentsExtraProps?.NavBar?.ref}
        isVisible={visible && !routeIsPlayerScreen(currentRoute)}
        menuPlugin={menuPlugin}
        navigationProps={navigationProps}
        {...R.omit(["ref"])(ComponentsExtraProps?.NavBar)}
      />
    </NavBarContainer>
  );

  const isTVOS = isApplePlatform();

  if (!appReady) {
    return null;
  }

  return (
    <LayoutContainer>
      <ScreenLayoutContextProvider>
        <Components.Background
          backgroundColor={backgroundColor}
          backgroundImageUrl={backgroundImageUrl}
          {...R.omit(["ref"])(ComponentsExtraProps?.Background)}
        >
          <ScreenDataContext.Provider value={navigator.data}>
            <PathnameContext.Provider value={navigator.currentRoute}>
              {!isTVOS ? navBarContainer : null}
              {navBarReady ? (
                /**
             Need to re-mount this component on each navigation event
             It's a hack to work around the fact that we don't support
             correct screen rendering in our navigator (shared currentRoute state)
             **/
                <ScreenContainer>{children}</ScreenContainer>
              ) : null}
              {/** Render NavBarContainer over ScreenContainer to let tvOS native
          focus manager works correctly to focus nav bar items first **/}
              {isTVOS ? navBarContainer : null}
            </PathnameContext.Provider>
          </ScreenDataContext.Provider>
        </Components.Background>
      </ScreenLayoutContextProvider>
    </LayoutContainer>
  );
};

export default Layout;
