import { resolveColor } from "..";
import { masterCellLogger } from "../../logger";

const loggerSpy = jest
  .spyOn(masterCellLogger, "warn")
  .mockImplementation(() => {});

describe("resolveColor", () => {
  const entry = {
    extensions: {
      color: "red",
      green_color: "green",
    },
  };

  beforeEach(() => {
    loggerSpy.mockClear();
  });

  it("resolve color with path", () => {
    const style = {
      color: "extensions.color",
    };

    expect(resolveColor(entry, style)).toEqual({
      color: entry.extensions.color,
    });
  });

  it("logs a warning when path is undefined", () => {
    const style = {
      color: "invalid_path",
    };

    expect(resolveColor(entry, style)).toEqual(style);

    expect(loggerSpy).toHaveBeenCalledWith(
      expect.objectContaining({
        message: "Cannot resolve property invalid_path from the entry.",
        data: {
          configurationValue: "invalid_path",
          entry,
        },
      })
    );
  });

  it("resolves any style prop with containing color", () => {
    const style = {
      backgroundColor: "extensions.color",
      width: "100%",
      borderColor: "extensions.green_color",
    };

    expect(resolveColor(entry, style)).toEqual({
      backgroundColor: "red",
      borderColor: "green",
      width: "100%",
    });
  });

  it("not replace color with in hex format", () => {
    const style = {
      color: "#000000",
    };

    expect(resolveColor(entry, style)).toEqual(style);
  });

  it("not replace color with in rgda format", () => {
    const style = {
      color: "rgba(0,0,0,0)",
    };

    expect(resolveColor(entry, style)).toEqual(style);
  });

  it("doesn't replace transparent as a color", () => {
    const style = {
      color: "transparent",
    };

    expect(resolveColor(entry, style)).toEqual(style);
  });

  it("return nil if style is nil", () => {
    const style = undefined;

    expect(resolveColor(entry, style)).toBeUndefined();
  });

  it("not modify style without color prop", () => {
    const style = {
      test: 1,
    };

    expect(resolveColor(entry, style)).toEqual(style);
  });

  it("not modify style with not existing path", () => {
    const style = {
      color: "not.exist.path",
    };

    expect(resolveColor(entry, style)).toEqual(style);
  });

  it("not modify style with empty path", () => {
    const style = {
      color: "",
    };

    expect(resolveColor(entry, style)).toEqual(style);
  });
});
