import * as React from "react";
import {
  Dimensions,
  Platform,
  StyleSheet,
  View,
  ViewStyle,
} from "react-native";
import { Edge, SafeAreaView } from "react-native-safe-area-context";

import { isTV } from "@applicaster/zapp-react-native-utils/reactUtils";
import { useIsTablet } from "@applicaster/zapp-react-native-utils/reactHooks";
import { PlayerDetails } from "./PlayerDetails";
import { playerDimesionsHack } from "./utils";
import { PathnameContext } from "../../Contexts/PathnameContext";
import { ScreenContextProvider } from "../../Contexts/ScreenContext";
import { ROUTE_TYPES } from "@applicaster/zapp-react-native-utils/navigationUtils/routeTypes";

import {
  AnimatedScrollModal,
  AnimatedVideoPlayerComponent,
  AnimationComponent,
  ComponentAnimationType,
} from "@applicaster/zapp-react-native-ui-components/Components/VideoModal/ModalAnimation";

type DimensionsT = {
  width: number | string;
  height: number | string | undefined;
  aspectRatio?: number;
};

type Configuration = {
  [key: string]: any;
  tablet_landscape_sidebar_width?: string;
  tablet_landscape_player_container_background_color?: string;
};

type Props = {
  entry: ZappEntry;
  style: ViewStyle & DimensionsT;
  containerStyle?: ViewStyle;
  inline?: boolean;
  docked?: boolean;
  pip?: boolean;
  isModal?: boolean;
  fullscreen?: boolean;
  isTabletPortrait?: boolean;
  children: (playerDimensions: DimensionsT) => React.ReactNode;
  configuration: Configuration;
};

const defaultStyles = StyleSheet.create({
  playerContainer: { position: "relative", alignSelf: "center", zIndex: 200 },
  playerDetails: { flex: 1, paddingTop: 20 },
  flex: { flex: 1 },
});

const orientationStyles = StyleSheet.create({
  landscape: { flexDirection: "row" },
  portrait: { flexDirection: "column" },
});

const directionStyles = (isTabletLandscape: boolean): ViewStyle => {
  if (isTabletLandscape) {
    return orientationStyles.landscape;
  }

  return orientationStyles.portrait;
};

const getScreenAspectRatio = () => {
  const { width, height } = Dimensions.get("screen");

  const longEdge = Math.max(width, height);
  const shortEdge = Math.min(width, height);

  return longEdge / shortEdge;
};

const safeAreaStyles = (
  configuration: Configuration,
  isTablet: boolean
): ViewStyle => {
  const tablet_landscape_player_container_background_color =
    configuration?.tablet_landscape_player_container_background_color;

  return {
    backgroundColor: isTablet
      ? tablet_landscape_player_container_background_color
      : "transparent",
  };
};

const getEdges = (isTablet: boolean, isInlineModal: boolean) => {
  if (isTablet) {
    return ["top"];
  }

  if (!isInlineModal && Platform.OS === "android") {
    return [];
  }

  return ["top"];
};

const getTabletWidth = (
  configuration: Configuration,
  dimensions: DimensionsT
) => {
  const tablet_landscape_sidebar_width =
    configuration?.tablet_landscape_sidebar_width;

  const { width } = dimensions;

  const sidebarWidth = Number(tablet_landscape_sidebar_width?.replace("%", ""));

  if (tablet_landscape_sidebar_width?.includes("%")) {
    return Number(width) * (1 - sidebarWidth / 100);
  }

  if (Number.isNaN(sidebarWidth)) {
    return Number(width) * 0.65;
  }

  return Number(width) - sidebarWidth;
};

export const PlayerWrapper = (props: Props) => {
  const {
    entry,
    style,
    containerStyle,
    inline,
    docked,
    isModal,
    children,
    isTabletPortrait,
    configuration,
    pip,
  } = props;

  const isTablet = useIsTablet();

  const isInlineModal = inline && isModal;

  const isTabletLandscape = !isTV() && isTablet && !isTabletPortrait;

  const tabletWidth = getTabletWidth(configuration, style);

  const baseDimensions: DimensionsT = React.useMemo(
    () => ({
      width:
        isInlineModal && !docked && isTabletLandscape ? tabletWidth : "100%",
      height: undefined,
    }),
    [isInlineModal, tabletWidth, docked]
  );

  const playerDimensions: DimensionsT = React.useMemo(
    () => ({
      ...baseDimensions,
      width: isInlineModal && docked ? undefined : baseDimensions.width,
      aspectRatio: !isInlineModal && !pip ? getScreenAspectRatio() : 16 / 9,
    }),
    [baseDimensions, isInlineModal, pip]
  );

  const containerDimensions: DimensionsT = React.useMemo(
    () => ({
      ...baseDimensions,
      aspectRatio:
        isInlineModal && docked ? undefined : playerDimensions.aspectRatio,
    }),
    [baseDimensions, isInlineModal, docked, playerDimensions.aspectRatio]
  );

  const WrapperView = React.useMemo(() => (isTV() ? View : SafeAreaView), []);

  const childrenStyles = React.useMemo(
    () => ({ ...playerDimensions, ...playerDimesionsHack }),
    [containerDimensions, playerDimesionsHack]
  );

  const pathname = `${ROUTE_TYPES.VIDEO_MODAL}/${entry?.id}`;

  return (
    <PathnameContext.Provider value={pathname}>
      <ScreenContextProvider pathname={pathname}>
        <WrapperView
          edges={getEdges(isTablet, isInlineModal) as readonly Edge[]}
          style={[
            safeAreaStyles(configuration, isTablet),
            style,
            playerDimesionsHack,
          ]}
        >
          <AnimationComponent
            animationType={ComponentAnimationType.moveUpComponent}
            additionalData={{ saveArea: true }}
            style={[directionStyles(isTabletLandscape), defaultStyles.flex]}
          >
            <PathnameContext.Provider value={pathname}>
              <ScreenContextProvider pathname={pathname}>
                <View
                  testID={`${entry?.id}-player-container`}
                  style={[
                    defaultStyles.playerContainer,
                    containerDimensions,
                    containerStyle,
                    playerDimesionsHack,
                  ]}
                >
                  <AnimationComponent
                    animationType={ComponentAnimationType.moveUpComponent}
                    style={isTabletLandscape ? defaultStyles.flex : undefined}
                    additionalData={{
                      useLayoutMeasure: isTabletLandscape,
                      disableAnimatedComponent: !isTabletLandscape,
                      resetAnimationValue: isTabletLandscape && docked,
                    }}
                  >
                    <AnimatedVideoPlayerComponent>
                      {children(childrenStyles)}
                    </AnimatedVideoPlayerComponent>
                  </AnimationComponent>
                </View>
                <AnimatedScrollModal>
                  {isInlineModal ? (
                    <AnimationComponent
                      animationType={ComponentAnimationType.componentFade}
                      style={defaultStyles.flex}
                    >
                      <PlayerDetails
                        configuration={configuration}
                        style={defaultStyles.playerDetails}
                        entry={entry}
                        isTabletLandscape={isTabletLandscape}
                      />
                    </AnimationComponent>
                  ) : (
                    <View style={defaultStyles.playerDetails} />
                  )}
                </AnimatedScrollModal>
              </ScreenContextProvider>
            </PathnameContext.Provider>
          </AnimationComponent>
        </WrapperView>
      </ScreenContextProvider>
    </PathnameContext.Provider>
  );
};
