import React from "react";
import { shallow } from "enzyme";
import { shallowToJson } from "enzyme-to-json";
import * as zappPipesRedux from "@applicaster/zapp-react-native-redux/ZappPipes";
import configureStore from "redux-mock-store";
import thunk from "redux-thunk";

import { zappPipesDataConnector } from "../index";

import { Hero } from "./Hero";

const mockStore = configureStore([thunk]);

const initialState = {
  zappPipes: {
    data: {
      next: "someNextFeedUrl",
      entry: [{ type: "video" }, { type: "article" }],
    },
  },
};

const store = mockStore(initialState);

const zappPipesSpy = jest
  .spyOn(zappPipesRedux, "loadPipesData")
  .mockImplementation(() => ({ type: "load_data", payload: "data" }));

const WrappedComponent = zappPipesDataConnector(Hero);

const componentProp = (data) => ({ data });

// tests need to be re-written due to TS refactor
describe.skip("zappPipesDataConnector", () => {
  beforeEach(() => {
    zappPipesSpy.mockClear();
  });

  describe("with a connected screen", () => {
    const data = { source: "", type: "" };
    const feedUrl = "feedUrlDs";

    it("loads the provided feed url", () => {
      expect(zappPipesDataConnector).toBeFunction();

      const wrapper = shallow(
        <WrappedComponent
          component={componentProp(data)}
          feedUrl={feedUrl}
          store={store}
        />
      ).dive();

      expect(shallowToJson(wrapper)).toMatchSnapshot();
      expect(wrapper.first().shallow()).toMatchSnapshot();

      expect(zappPipesSpy).toHaveBeenCalledWith(feedUrl, {
        clearCache: false,
        loadLocalFavorites: false,
        silentRefresh: true,
      });
    });
  });

  describe("with standard dataSource", () => {
    const data = {
      source: "dataSourceUrl",
      type: "APPLICASTER_ATOM_FEED",
    };

    it("is a HoC", () => {
      expect(zappPipesDataConnector).toBeFunction();

      const wrapper = shallow(
        <WrappedComponent component={componentProp(data)} store={store} />
      ).dive();

      expect(shallowToJson(wrapper)).toMatchSnapshot();
      expect(wrapper.first().shallow()).toMatchSnapshot();

      expect(zappPipesSpy).toHaveBeenCalledWith(data.source, {
        clearCache: false,
        loadLocalFavorites: false,
        silentRefresh: true,
      });
    });
  });

  describe("with clear_cache_on_reload", () => {
    const data = {
      source: "dataSourceUrl",
      type: "APPLICASTER_ATOM_FEED",
    };

    const rules = {
      clear_cache_on_reload: "true",
    };

    it("calls clearCache true and silentRefresh with false", () => {
      expect(zappPipesDataConnector).toBeFunction();

      const wrapper = shallow(
        <WrappedComponent component={{ data, rules }} store={store} />
      ).dive();

      expect(shallowToJson(wrapper)).toMatchSnapshot();
      expect(wrapper.first().shallow()).toMatchSnapshot();

      expect(zappPipesSpy).toHaveBeenCalledWith(data.source, {
        clearCache: true,
        loadLocalFavorites: false,
        silentRefresh: false,
      });
    });
  });

  describe("with Applicaster Collection Id", () => {
    const data = {
      source: "1234",
      type: "APPLICASTER_COLLECTION",
    };

    it("is a HoC", () => {
      expect(zappPipesDataConnector).toBeFunction();

      const wrapper = shallow(
        <WrappedComponent component={componentProp(data)} store={store} />
      ).dive();

      expect(shallowToJson(wrapper)).toMatchSnapshot();
      expect(wrapper.first().shallow()).toMatchSnapshot();

      expect(zappPipesSpy).toHaveBeenCalledWith(
        "applicaster://fetchData?type=APPLICASTER_COLLECTION&collectionId=1234",
        { clearCache: false, loadLocalFavorites: false, silentRefresh: true }
      );
    });
  });

  describe("with Applicaster Category Id", () => {
    const data = {
      source: "1234",
      type: "APPLICASTER_CATEGORY",
    };

    it("is a HoC", () => {
      expect(zappPipesDataConnector).toBeFunction();

      const wrapper = shallow(
        <WrappedComponent component={componentProp(data)} store={store} />
      ).dive();

      expect(shallowToJson(wrapper)).toMatchSnapshot();
      expect(wrapper.first().shallow()).toMatchSnapshot();

      expect(zappPipesSpy).toHaveBeenCalledWith(
        "applicaster://fetchData?type=APPLICASTER_CATEGORY&categoryId=1234&platform=ios",
        {
          clearCache: false,
          loadLocalFavorites: false,
          silentRefresh: true,
        }
      );
    });
  });

  describe("with Favourites datasource", () => {
    const data = {
      source: null,
      type: "FAVOURITES",
    };

    it("calls loadPipesData with favourites option", () => {
      const wrapper = shallow(
        <WrappedComponent component={componentProp(data)} store={store} />
      ).dive();

      expect(shallowToJson(wrapper)).toMatchSnapshot();
      expect(wrapper.first().shallow()).toMatchSnapshot();

      expect(zappPipesSpy).toHaveBeenCalledWith("FAVOURITES", {
        clearCache: true,
        loadLocalFavorites: true,
        silentRefresh: true,
      });
    });
  });

  describe("getDataSourceUrl", () => {
    it("returns null if data prop is empty", () => {
      const wrapper = shallow(
        <WrappedComponent component={componentProp({})} store={store} />
      ).dive();

      expect(wrapper.first().shallow().instance().getDataSourceUrl()).toEqual({
        dataSourceUrl: null,
        type: null,
      });
    });

    it("returns null if data.source is falsy", () => {
      const wrapper = shallow(
        <WrappedComponent
          component={componentProp({ source: null })}
          store={store}
        />
      ).dive();

      expect(wrapper.first().shallow().instance().getDataSourceUrl()).toEqual({
        dataSourceUrl: null,
        type: null,
      });
    });
  });

  describe("reloadDataSource", () => {
    const data = {
      source: "1234",
      type: "APPLICASTER_CATEGORY",
    };

    it("reloads the app's datasource", () => {
      store.clearActions();

      const wrapper = shallow(
        <WrappedComponent component={componentProp(data)} store={store} />
      ).dive();

      const actionsBefore = store.getActions();
      expect(actionsBefore).toMatchSnapshot();

      const heroProps = wrapper.first().shallow().props();

      heroProps.reloadData();

      const actionsAfter = store.getActions();
      expect(actionsAfter).toMatchSnapshot();
    });
  });

  describe("Load staticFeed", () => {
    const data = {
      source: null,
    };

    it("should load the static feed if pipes url is missing and static feed getter is present", () => {
      store.clearActions();

      const getStaticComponentFeed = jest.fn(() => ({
        id: "test-feed",
        entry: [{ id: "test-entry", title: "test entry" }],
      }));

      const wrapper = shallow(
        <WrappedComponent
          component={{ data, getStaticComponentFeed }}
          store={store}
        />
      ).dive();

      const heroProps = wrapper.first().shallow().props();

      expect(heroProps).toBe(true);
    });
  });
});
